package auth

import (
	"context"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"net/http"
	"net/url"
	"path"
	"time"

	lfcc "git.loafle.net/commons_go/config"
	"git.loafle.net/commons_go/logging"
	"git.loafle.net/overflow/overflow_probes/central/client"
	"git.loafle.net/overflow/overflow_probes/commons"
	"git.loafle.net/overflow/overflow_probes/config"
)

const (
	noAuthConfigFileName     = "noauth.json"
	noAuthHeaderNoAuthID     = "overFlow-NoAuth-ID"
	noAuthHeaderNoAuthRegist = "overFlow-NoAuth-Regist"
	noAuthHeaderSetNoAuthID  = "overFlow-Set-NoAuth-ID"
	probeConfigFileName      = "probe.json"
)

type AuthHandler interface {
	commons.Handler
}

type authHandlers struct {
	configDir        string
	noAuthConfigPath string
	noAuthConfig     config.NoAuthProbeConfig
	entryURL         string
	c                client.Client
	probeConfigPath  string
	probeConfig      config.ProbeConfig
}

type NoAuthProbe struct {
	APIKey      string `json:"apiKey"`
	Description string `json:"description"`
}

type NoAuthDescription struct {
	Host    Host    `json:"host"`
	Network Network `json:"network"`
}

func New(configDir string) AuthHandler {
	h := &authHandlers{
		configDir: configDir,
	}

	var err error
	var centralURL *url.URL
	var noAuthEntryPoint string
	var ok bool
	if centralURL, err = url.Parse(config.Config.Central.URL); nil != err {
		logging.Logger.Error(fmt.Sprintf("Auth: Central URL[%s] is not valid error[%v]", config.Config.Central.URL, err))
		return nil
	}

	if noAuthEntryPoint, ok = config.Config.Central.EntryPoints["noauth"]; !ok {
		logging.Logger.Error("Auth: NoAuth entry point of Central is not exist")
		return nil
	}

	centralURL.Path = path.Join(centralURL.Path, noAuthEntryPoint)
	h.entryURL = centralURL.String()

	h.noAuthConfigPath = path.Join(configDir, noAuthConfigFileName)
	h.probeConfigPath = path.Join(configDir, probeConfigFileName)

	conf := lfcc.New()
	if lfcc.Exists(h.noAuthConfigPath) {
		if err = conf.Load(&h.noAuthConfig, h.noAuthConfigPath); nil != err {
			logging.Logger.Error(fmt.Sprintf("Auth: Loading of NoAuth config file[%s] failed error[%v]", h.noAuthConfigPath, err))
		}
	} else {
		if err = lfcc.Save(h.noAuthConfig, h.noAuthConfigPath, true); nil != err {
			logging.Logger.Error(fmt.Sprintf("Auth: Saving of NoAuth config file[%s] failed error[%v]", h.noAuthConfigPath, err))
		}
	}

	return h
}

func (h *authHandlers) Listen() error {
	var err error
	isRegist := true

	h.c = client.New()
	h.c.OnNotify("NoAuthProbeService.accept", h.onNoAuthProbeAccept)
	h.c.OnNotify("NoAuthProbeService.deny", h.onNoAuthProbeDeny)

	header := http.Header{}

	if "" != h.noAuthConfig.TempKey {
		isRegist = false
		header[noAuthHeaderNoAuthID] = []string{h.noAuthConfig.TempKey}
	} else {
		var enc string
		if enc, err = h.getRegistHeader(); nil != err {
			return err
		}
		header[noAuthHeaderNoAuthRegist] = []string{enc}
	}
	var res *http.Response
	if res, err = h.c.Dial(h.entryURL, header, 4096, 4096); nil != err {
		return err
	}

	if isRegist {
		h.noAuthConfig.TempKey = res.Header.Get(noAuthHeaderSetNoAuthID)
		if err = lfcc.Save(h.noAuthConfig, h.noAuthConfigPath, true); nil != err {
			return err
		}
	}

	for {

	}

	return nil
}

func (h *authHandlers) onNoAuthProbeAccept(method string, params interface{}) {
	var err error
	ps := params.([]string)
	probeID := ps[0]

	if lfcc.Exists(h.probeConfigPath) {
		if err = lfcc.Load(&h.probeConfig, h.probeConfigPath); nil != err {
			logging.Logger.Error(fmt.Sprintf("Auth: Loading of Probe config file[%s] failed error[%v]", h.probeConfigPath, err))
		}
	}

	h.probeConfig.ID = probeID
	if err = lfcc.Save(h.probeConfig, h.probeConfigPath, true); nil != err {
		logging.Logger.Error(fmt.Sprintf("Auth: Saving of Probe config file[%s] failed error[%v]", h.probeConfigPath, err))
	}

}

func (h *authHandlers) onNoAuthProbeDeny(method string, params interface{}) {
	h.noAuthConfig.DenyDate = time.Now()
	if err := lfcc.Save(h.noAuthConfig, h.noAuthConfigPath, true); nil != err {
		logging.Logger.Error(fmt.Sprintf("Auth: Saving of NoAuth config file[%s] failed error[%v]", h.noAuthConfigPath, err))
	}

}

func (h *authHandlers) getRegistHeader() (string, error) {
	var err error
	nap := NoAuthProbe{
		APIKey: config.Config.Domain.APIKey,
	}
	nad := NoAuthDescription{}

	if err = getHost(&nad.Host); nil != err {
		return "", err
	}

	if err = getNetwork(&nad.Network); nil != err {
		return "", err
	}

	var buf []byte
	if buf, err = json.Marshal(nad); nil != err {
		return "", err
	}
	nap.Description = string(buf)

	if buf, err = json.Marshal(nap); nil != err {
		return "", err
	}

	logging.Logger.Debug(fmt.Sprintf("%s", string(buf)))

	enc := base64.StdEncoding.EncodeToString(buf)

	return enc, nil
}

func (h *authHandlers) listen() {
	// 1. regist
	// 2. wait for accept auth
}

func (h *authHandlers) Shutdown(ctx context.Context) error {

	return nil
}
